@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2016	PgPool Global Development Group
+ * Copyright (c) 2003-2017	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -52,7 +52,7 @@
 
 #define MAX_FILE_LINE_LEN    512
 
-extern char pcp_conf_file[POOLMAXPATHLEN+1]; /* global variable defined in main.c holds the path for pcp.conf */
+extern char *pcp_conf_file; /* global variable defined in main.c holds the path for pcp.conf */
 volatile sig_atomic_t pcp_worker_wakeup_request = 0;
 PCP_CONNECTION* volatile pcp_frontend = NULL;
 
@@ -86,7 +86,6 @@ static void pcp_worker_will_go_down(int code, Datum arg);
 static void  do_pcp_flush(PCP_CONNECTION *frontend);
 static void  do_pcp_read(PCP_CONNECTION *pc, void *buf, int len);
 
-
 /* 
  * main entry pont of pcp worker child process
  */
@@ -502,14 +501,14 @@ static int pool_detach_node(int node_id, bool gracefully)
 {
 	if (!gracefully)
 	{
-		degenerate_backend_set_ex(&node_id, 1, true, false, true, 0);
+		degenerate_backend_set_ex(&node_id, 1, REQ_DETAIL_SWITCHOVER | REQ_DETAIL_CONFIRMED, true, false);
 		return 0;
 	}
 
 	/* Check if the NODE DOWN can be executed on
 	 * the given node id.
 	 */
-	degenerate_backend_set_ex(&node_id, 1, true, true, true, 0);
+	degenerate_backend_set_ex(&node_id, 1, REQ_DETAIL_SWITCHOVER | REQ_DETAIL_CONFIRMED, true, true);
 
 	/*
 	 * Wait until all frontends exit
@@ -529,7 +528,7 @@ static int pool_detach_node(int node_id, bool gracefully)
 	/*
 	 * Now all frontends have gone. Let's do failover.
 	 */
-	degenerate_backend_set_ex(&node_id, 1, true, false, true, 0);
+	degenerate_backend_set_ex(&node_id, 1, REQ_DETAIL_SWITCHOVER | REQ_DETAIL_CONFIRMED, false, true);
 
 	/*
 	 * Wait for failover completed.
@@ -556,7 +555,7 @@ static int pool_promote_node(int node_id, bool gracefully)
 {
 	if (!gracefully)
 	{
-		promote_backend(node_id, false);	/* send promote request */
+		promote_backend(node_id, REQ_DETAIL_CONFIRMED);	/* send promote request */
 		return 0;
 	}
 
@@ -576,7 +575,7 @@ static int pool_promote_node(int node_id, bool gracefully)
 	/*
 	 * Now all frontends have gone. Let's do failover.
 	 */
-	promote_backend(node_id, false);		/* send promote request */
+	promote_backend(node_id, REQ_DETAIL_CONFIRMED);		/* send promote request */
 
 	/*
 	 * Wait for failover completed.
@@ -811,8 +810,10 @@ inform_node_info(PCP_CONNECTION *frontend,char *buf)
 	char port_str[6];
 	char status[2];
 	char weight_str[20];
+	char role_str[10];
 	char code[] = "CommandComplete";
 	BackendInfo *bi = NULL;
+	SERVER_ROLE role;
 
 	node_id = atoi(buf);
 
@@ -830,6 +831,22 @@ inform_node_info(PCP_CONNECTION *frontend,char *buf)
 	snprintf(port_str, sizeof(port_str), "%d", bi->backend_port);
 	snprintf(status, sizeof(status), "%d", bi->backend_status);
 	snprintf(weight_str, sizeof(weight_str), "%f", bi->backend_weight);
+
+	if (STREAM)
+	{
+		if (Req_info->primary_node_id == node_id)
+			role = ROLE_PRIMARY;
+		else
+			role = ROLE_STANDBY;
+	}
+	else
+	{
+		if (Req_info->master_node_id == node_id)
+			role = ROLE_MASTER;
+		else
+			role = ROLE_SLAVE;
+	}
+	snprintf(role_str, sizeof(role_str), "%d", role);
 	
 	pcp_write(frontend, "i", 1);
 	wsize = htonl(sizeof(code) +
@@ -837,6 +854,7 @@ inform_node_info(PCP_CONNECTION *frontend,char *buf)
 				  strlen(port_str)+1 +
 				  strlen(status)+1 +
 				  strlen(weight_str)+1 +
+				  strlen(role_str)+1 +
 				  sizeof(int));
 	pcp_write(frontend, &wsize, sizeof(int));
 	pcp_write(frontend, code, sizeof(code));
@@ -844,6 +862,8 @@ inform_node_info(PCP_CONNECTION *frontend,char *buf)
 	pcp_write(frontend, port_str, strlen(port_str)+1);
 	pcp_write(frontend, status, strlen(status)+1);
 	pcp_write(frontend, weight_str, strlen(weight_str)+1);
+
+	pcp_write(frontend, role_str, strlen(role_str)+1);
 	do_pcp_flush(frontend);
 }
 
@@ -910,7 +930,7 @@ process_attach_node(PCP_CONNECTION *frontend,char *buf)
 			(errmsg("PCP: processing attach node"),
 			 errdetail("attaching Node ID %d", node_id)));
 
-	send_failback_request(node_id,true, false);
+	send_failback_request(node_id,true, REQ_DETAIL_CONFIRMED);
 
 	pcp_write(frontend, "c", 1);
 	wsize = htonl(sizeof(code) + sizeof(int));
@@ -1059,7 +1079,7 @@ process_promote_node(PCP_CONNECTION *frontend, char *buf, char tos)
 		
 	}
 
-	if (node_id == PRIMARY_NODE_ID)
+	if (node_id == REAL_PRIMARY_NODE_ID)
 	{
 		ereport(FATAL,
 				(errmsg("invalid pgpool mode for process recovery request"),
@@ -1289,4 +1309,3 @@ static void pcp_worker_will_go_down(int code, Datum arg)
 	POOL_SETMASK(&UnBlockSig);
 	
 }
-
